#include "widgetsimu.h"
#include "sailboat.h"


extern sailboat boat1;

WidgetSimu::WidgetSimu(QWidget *parent) :
    QGLWidget(parent)
{  Zoom=0.5;
    zCam=1000;
    xCam=-1100;

}


void WidgetSimu::Start()
{   if (boat1.dt==0)  boat1.dt=0.02; else boat1.dt=0;
}


void WidgetSimu::initializeGL()
{
    glEnable(GL_LIGHTING);
    glEnable(GL_LIGHT0);
    glEnable(GL_DEPTH_TEST);
    glEnable(GL_NORMALIZE);
    glClearColor(0.9,0.9,1,1);  //arrire plan

            static const GLfloat lightPos[4] = { 5.0f, 5.0f, 10.0f, 1.0f };
            glLightfv(GL_LIGHT0, GL_POSITION, lightPos);
            glEnable(GL_LIGHTING);
            xCam=-90; yCam=0;  zCam=20;
            glEnable(GL_COLOR_MATERIAL);
}

void WidgetSimu::paintGL()
{
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
        glLoadIdentity();
        glTranslated(0.0,-2.0, -40.0);
        glRotated(xCam, 1.0, 0.0, 0.0);
        glRotated(yCam, 0.0, 1.0, 0.0);
        glRotated(zCam, 0.0, 0.0, 1.0);
        glScalef( Zoom, Zoom, Zoom);
        glPushMatrix();
          DrawMer();
          DrawBouees();
        glPopMatrix();
        DrawSailboat();
}



void WidgetSimu::resizeGL(int width, int height)
{
    int side = qMin(width, height);
    glViewport((width - side) / 2, (height - side) / 2, side, side);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glFrustum(-1.0, +1.0, -1.0, 1.0, 5.0, 60.0);
    glMatrixMode(GL_MODELVIEW);
}

void WidgetSimu::mousePressEvent(QMouseEvent *event)
{   lastPos = event->pos();
    boat1.x = 0.0;  boat1.y = 0.0;
}

void WidgetSimu::mouseMoveEvent(QMouseEvent *event)
{   int dx = event->x() - lastPos.x();
    int dy = event->y() - lastPos.y();
    if (event->buttons() & Qt::LeftButton)
    {   xCam=xCam + dy;
        yCam=yCam + dx;
    }
    else if (event->buttons() & Qt::RightButton) {
        xCam=xCam + dy;
        zCam=zCam + dx;
    }
    lastPos = event->pos();
}

void WidgetSimu::enterEvent(QEvent *event)
{       setFocus();
        grabKeyboard();
}

void WidgetSimu::keyPressEvent(QKeyEvent *event)
{   switch (event->key())
    {
       case Qt::Key_Space: boat1.x = 0.0;      break;
       case Qt::Key_Plus:  Zoom=Zoom*1.1;   break;
       case Qt::Key_Minus: Zoom=Zoom/1.1;  break;
       case Qt::Key_Up:    boat1.deltavmax=fmax(0.0,boat1.deltavmax-0.05); break;
       case Qt::Key_Down:  boat1.deltavmax=fmin(1.2,boat1.deltavmax+0.05);  break;
       case Qt::Key_Right: boat1.deltag=fmin(boat1.deltag+0.05,0.5);  break;
       case Qt::Key_Left:  boat1.deltag=fmax(boat1.deltag-0.05,-0.5); break;
    }
}

void WidgetSimu::DrawMer()
{       glBegin(GL_POLYGON);
           glColor3f(0.5,0.5,1);
           glVertex3f(-100000,-100000,0);
           glVertex3f(100000,-100000,0);
           glVertex3f(100000,100000,0);
           glVertex3f(-100000,100000,0);
        glEnd();
}

void WidgetSimu::DrawBouees()
{   for (int k=1;k<3;k++)
    {  glTranslatef(10,0, 0);
       glColor3f(1, 0.0, 0.0);
       GLUquadricObj* q=gluNewQuadric();
       gluQuadricDrawStyle(q,GLU_FILL);
       gluSphere(q,1.0,10,10);
    }
}



void WidgetSimu::DrawSailboat()

{       glTranslated(boat1.x,boat1.y,0.0);
        glPushMatrix();

        // ***************************************
        //       WIND
        // ***************************************

                    glBegin(GL_LINES);  //Fleche vent
                            double a=1;
                            glColor3f(0.7,0.1,0);             glVertex3f(3,14,13);
                            glVertex3f(3,14-a,13);            glVertex3f(3,14-a,13);
                            glVertex3f(3,14-a,14);            glVertex3f(3,14-a,14);
                            glVertex3f(3,12.5-a,12.5);        glVertex3f(3,12.5-a,12.5);
                            glVertex3f(3,14-a,11);            glVertex3f(3,14-a,11);
                            glVertex3f(3,14-a,12);            glVertex3f(3,14-a,12);
                            glVertex3f(3,14,12);              glVertex3f(3,14,12);
                            glVertex3f(3,14,13);
                    glEnd();


        glPopMatrix();
        glRotatef(boat1.theta*RadToDeg,0.0,0.0,1.0);
        glPushMatrix();
        glRotatef(boat1.phi*RadToDeg,1,0,0);

        // ***************************************
        //       COQUE
        // ***************************************

                                            {   glBegin(GL_QUADS);   //plancher
                                                   glVertex3f(-1,-1.5,0.1);        glVertex3f(3,-1.5, 0.1);
                                                   glVertex3f(3,1.5,0.1);          glVertex3f(-1,1.5,0.1);
                                                   glVertex3f(4.5,-0.75, 0.1);     glVertex3f(5.06,-0.375, 0.1);
                                                   glVertex3f(5.25,0, 0.1);        glVertex3f(5.06,0.375, 0.1);
                                                   glVertex3f(4.5,0.75, 0.1);
                                                glEnd();
                                                // ---------------------------- dessus avant ----------------------------------------------------------------------
                                                glTranslatef(0.01,0,0.01);
                                                glBegin(GL_POLYGON);  //dessus avant
                                                   glColor3f(0.8,0.2,0.2);
                                                   glVertex3f(3,-2,1.0);
                                                   glVertex3f(3.1,-1.9,1.01);
                                                   glVertex3f(3.75,-1.6, 1.01);
                                                   glVertex3f(5.0,-1,1.01);
                                                   glVertex3f(5.75,-0.5,1.01);
                                                   glVertex3f(6.0, 0, 1.01);
                                                   glVertex3f(5.75,0.5,1.01);
                                                   glVertex3f(5.0,1,1.01);
                                                   glVertex3f(3.75,1.6,1.01);
                                                   glVertex3f(3.1,1.9,1.01);
                                                   glVertex3f(3,2,1.01);
                                                glEnd();

                                                //--------------------------- faces latrales ---------------------------------------------------------------------
                                                glTranslatef(-0.01,0,0);
                                                glBegin(GL_POLYGON); //faces latrales
                                                    glColor3f(0,1,0);
                                                    glVertex3f(-1,-1.5,0);       glVertex3f(-1, -1.9, 0.5);
                                                    glVertex3f(-1,-2,1.0);       glVertex3f(3,-2,1.0);
                                                    glVertex3f(3, -1.9, 0.5);    glVertex3f(3,-1.5,0);
                                                    glVertex3f(-1,-1.5,0);
                                                    glColor3f(0,1,0);
                                                    glVertex3f(-1,1.5,0);        glVertex3f(-1, 1.9, 0.5);
                                                    glVertex3f(-1,2,1.0);        glVertex3f(3,2,1.0);
                                                    glVertex3f(3, 1.9, 0.5);     glVertex3f(3,1.5,0);
                                                    glVertex3f(-1,1.5,0);
                                                glEnd();

                                                //--------------------------- banc ----------------------------------------------------------------------------------
                                                glTranslatef(0,-0.01,0);
                                                glBegin(GL_QUADS);              //banc
                                                    glColor3f(1,1,1);
                                                    glVertex3f(-1,2,1.0);         glVertex3f(-1, 1.3, 1.0);
                                                    glVertex3f(3, 1.3, 1.0);      glVertex3f(3,2,1.0);
                                                    glVertex3f(-1, 1.3, 1.0);     glVertex3f(-1, 1.1, 4*1.0/5);
                                                    glVertex3f(3, 1.1, 4*1.0/5);  glVertex3f(3, 1.3, 1.0);
                                                    glVertex3f(-1, 1.1, 4*1.0/5); glVertex3f(-1, 0.9, 3*1.0/5);
                                                    glVertex3f(3, 0.9, 3*1.0/5);  glVertex3f(3, 1.1, 4*1.0/5);
                                                    glVertex3f(-1, 0.9, 3*1.0/5); glVertex3f(-1,0.8,0);
                                                    glVertex3f(3,0.8,0);          glVertex3f(3, 0.9, 3*1.0/5);
                                                glEnd();
                                                glTranslatef(0,0.01,0);

                                                // -------------------------------- autre banc ----------------------------------------------------------------
                                                glBegin(GL_QUADS);              //banc
                                                    glColor3f(1,1,1);
                                                    glVertex3f(-1,-2,1.0);        glVertex3f(-1, -1.3, 1.0);
                                                    glVertex3f(3, -1.3, 1.0);     glVertex3f(3,-2,1.0);
                                                    glVertex3f(-1, -1.3, 1.0);    glVertex3f(-1, -1.1, 4*1.0/5);
                                                    glVertex3f(3, -1.1, 4*1.0/5); glVertex3f(3, -1.3, 1.0);
                                                    glVertex3f(-1, -1.1, 4*1.0/5);glVertex3f(-1, -0.9, 3*1.0/5);
                                                    glVertex3f(3, -0.9, 3*1.0/5); glVertex3f(3, -1.1, 4*1.0/5);
                                                    glVertex3f(-1, -0.9, 3*1.0/5);glVertex3f(-1,-0.8,0);
                                                    glVertex3f(3,-0.8,0);         glVertex3f(3, -0.9, 3*1.0/5);
                                                glEnd();

                                                glTranslatef(0,-0.01,0);

                                                // ---------------------- face avant mat --------------------------------
                                                glBegin(GL_POLYGON); //face "avant mat"
                                                    glColor3f(0,0,1);
                                                    glNormal3f(-1,0,0);
                                                    glVertex3f(3+0,-1.5+0.4,0);
                                                    glVertex3f(3+0, -1.9+0.4, 0.5);
                                                    glVertex3f(3+0,-2+0.4,1.0);
                                                    glVertex3f(3+0,2-0.4,1.0);
                                                    glVertex3f(3+0, 1.9-0.4, 0.5);
                                                    glVertex3f(3+0,1.5-0.4,0);
                                                    glVertex3f(3+0,-1.5+0.4,0);
                                                glEnd();

                                                // ------------------------ face arrire -----------------------------------------------
                                                glTranslatef(-0.01,0,0);
                                                glBegin(GL_POLYGON); //face arrire
                                                    glColor3f(0.5,1,0);
                                                    glNormal3f(-1,0,0);
                                                    glVertex3f(-1,-1.5,0);
                                                    glVertex3f(-1, -1.9, 0.5);
                                                    glVertex3f(-1,-2,1.0);
                                                    glVertex3f(-1,2,1.0);
                                                    glVertex3f(-1, 1.9, 0.5);
                                                    glVertex3f(-1,1.5,0);
                                                    glVertex3f(-1,-1.5,0);
                                                glEnd();
                                                glTranslatef(0.01,-0.01,0);

                                                // ----------------------------- face avant ------------------------------------------
                                                glBegin(GL_POLYGON);
                                                   glVertex3f(3,-2,1.0);           glVertex3f(3.1,-1.7,1.0);
                                                   glVertex3f(3.75,-1.5, 1.0);     glVertex3f(5.0,-1,1.0);
                                                   glVertex3f(5.75,-0.5,1.0);      glVertex3f(6.0, 0, 1.0);
                                                   glVertex3f(5.25,0, 0);          glVertex3f(5.06,-0.375, 0);
                                                   glVertex3f(4.5,-0.75, 0);       glVertex3f(3,-1.5,0);
                                                   glVertex3f(3, -1.9, 0.5);       glVertex3f(3,-2,1.0);
                                                   glVertex3f(3,2,1.0);            glVertex3f(3.1,1.7,1.0);
                                                   glVertex3f(3.75,1.5, 1.0);      glVertex3f(5.0,1,1.0);
                                                   glVertex3f(5.75,0.5,1.0);       glVertex3f(6.0, 0, 1.0);
                                                   glVertex3f(5.25,0, 0);          glVertex3f(5.06,0.375, 0);
                                                   glVertex3f(4.5,0.75, 0);        glVertex3f(3,1.5,0);
                                                   glVertex3f(3, 1.9, 0.5);        glVertex3f(3,2,1.0);
                                                glEnd();
                                            }



        glPopMatrix();
        glPushMatrix();
             glTranslatef(-1,0,0);
             glRotatef(boat1.deltag*RadToDeg,0,0,1);

             // ***************************************
             //       RUDDER
             // ***************************************
                                             {   glColor3f(0,0,1);
                                                 glBegin(GL_POLYGON);
                                                     glVertex3f(0,-0.05,1);       glVertex3f(0,-0.05,0);      glVertex3f(-0.6,-0.05,0);
                                                     glVertex3f(-0.3,-0.05,1);    glVertex3f(0,-0.05,1);      glVertex3f(0,0.05,1);
                                                     glVertex3f(0,0.05,0);        glVertex3f(-0.6,0.05,0);    glVertex3f(-0.3,0.05,1);
                                                     glVertex3f(0,0.05,1);
                                                     glVertex3f(0,-0.05,1);       glVertex3f(0,0.05,1);       glVertex3f(0,-0.05,0);
                                                     glVertex3f(0,0.05,0);        glVertex3f(-0.6,-0.05,0);   glVertex3f(-0.6,0.05,0);
                                                     glVertex3f(-0.3,-0.05,1);    glVertex3f(-0.3,0.05,1);    glVertex3f(0,-0.05,1);
                                                     glVertex3f(0,0.05,1);
                                                 glEnd();
                                                 glDisable(GL_TEXTURE_2D);
                                                 glTranslatef(-0.6,0,1);
                                                 glRotatef(90,0,1,0);
                                                 GLUquadricObj*q1=gluNewQuadric();  //barre
                                                 gluQuadricOrientation(q1, GLU_OUTSIDE);
                                                 gluQuadricTexture(q1, GL_TRUE);
                                                 gluCylinder(q1,0.05,0.05,3, 10,10);
                                                 gluDeleteQuadric(q1);
                                             //        glDisable(GL_TEXTURE_2D);
                                             }





        glPopMatrix();
        glPushMatrix();
            glTranslatef(5,0,0);
            glRotatef(boat1.deltav*RadToDeg,0,0,1);

           // ***************************************
           //       SAIL
           // ***************************************
;
                                            {       glColor3f(0.9,0.9,0.9);
                                                    GLUquadricObj*q1=gluNewQuadric();  //mt
                                                    gluQuadricOrientation(q1, GLU_OUTSIDE);
                                                    gluQuadricTexture(q1, GL_TRUE);
                                                    gluCylinder(q1,0.08,0.08,14, 10,10);
                                                    gluDeleteQuadric(q1);
                                                    glDisable(GL_TEXTURE_2D);


                                                    glColor3f(1,1,0);
                                                        float a=-atan(boat1.fv/500);  // courbure de la voile
                                                        glEnable(GL_TEXTURE_2D);
                                                        glBegin(GL_TRIANGLE_FAN);
                                                            glVertex3f(0,0,2);
                                                            glVertex3f(0,0,12);
                                                            glVertex3f(-1,a*1.5,10);
                                                            glVertex3f(-2,a*2,8);
                                                            glVertex3f(-3,a*2,6);
                                                            glVertex3f(-4,a*1.5,4);
                                                            glVertex3f(-5,0,2);
                                                        glEnd();

                                                        glDisable(GL_BLEND);
                                                        glDisable(GL_TEXTURE_2D);
                                                        glTranslatef(0,0,2);
                                                        glRotatef(-90,0,1,0);
                                                        glEnable(GL_TEXTURE_2D);         //bome
                                                        glBindTexture(GL_TEXTURE_2D,5);


                                                        q1=gluNewQuadric();
                                                        gluQuadricOrientation(q1, GLU_OUTSIDE);
                                                        gluQuadricTexture(q1, GL_TRUE);
                                                        gluCylinder(q1,0.1,0.1,5.5, 10,10);
                                                        gluDeleteQuadric(q1);
                                                        glDisable(GL_TEXTURE_2D);
                                            }



        glPopMatrix();
}



void WidgetSimu::Clock()
{
    boat1.Clock();
    DrawSailboat();
    updateGL();
}





